'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.login = exports.resetLogin = exports.LOGIN_REJECTED = exports.LOGIN_FULFILLED = exports.LOGIN_STARTED = exports.RESET_LOGIN = exports.AUTH_RENEW_REJECTED = exports.AUTH_RENEW_FULFILLED = undefined;

var _cspace = require('./cspace');

var _account = require('./account');

var _prefs = require('./prefs');

var _reducers = require('../reducers');

var AUTH_RENEW_FULFILLED = exports.AUTH_RENEW_FULFILLED = 'AUTH_RENEW_FULFILLED';
var AUTH_RENEW_REJECTED = exports.AUTH_RENEW_REJECTED = 'AUTH_RENEW_REJECTED';
var RESET_LOGIN = exports.RESET_LOGIN = 'RESET_LOGIN';
var LOGIN_STARTED = exports.LOGIN_STARTED = 'LOGIN_STARTED';
var LOGIN_FULFILLED = exports.LOGIN_FULFILLED = 'LOGIN_FULFILLED';
var LOGIN_REJECTED = exports.LOGIN_REJECTED = 'LOGIN_REJECTED';

var resetLogin = exports.resetLogin = function resetLogin(username) {
  return {
    type: RESET_LOGIN,
    meta: {
      username: username
    }
  };
};

var renewAuth = function renewAuth(username, password) {
  return function (dispatch) {
    var session = (0, _cspace.createSession)(username, password);

    return session.login().then(function (response) {
      dispatch((0, _cspace.setSession)(session));

      return dispatch({
        type: AUTH_RENEW_FULFILLED,
        payload: response,
        meta: {
          username: username
        }
      });
    }).catch(function (error) {
      dispatch({
        type: AUTH_RENEW_REJECTED,
        payload: error,
        meta: {
          username: username
        }
      });

      throw error;
    });
  };
};

var login = exports.login = function login(config, username, password) {
  return function (dispatch, getState) {
    var prevUsername = (0, _reducers.getUserUsername)(getState());

    dispatch((0, _prefs.savePrefs)());

    dispatch({
      type: LOGIN_STARTED,
      meta: {
        username: username
      }
    });

    return dispatch(renewAuth(username, password)).then(function () {
      return dispatch((0, _account.readAccountPerms)(config, username));
    }).then(function () {
      return dispatch((0, _prefs.loadPrefs)(username));
    }).then(function () {
      return dispatch({
        type: LOGIN_FULFILLED,
        meta: {
          prevUsername: prevUsername,
          username: username
        }
      });
    }).catch(function (error) {
      return dispatch({
        type: LOGIN_REJECTED,
        payload: error,
        meta: {
          username: username
        }
      });
    });
  };
};